import pygame, sys
from pygame.locals import *

BLANK = 0
BLOCK = 1
FOOD = 2
WARP = 3
CURVE = 4

RIGHT = 0
DOWN = 1
LEFT = 2
UP = 3

MAP_WIDTH = 20
MAP_HEIGHT = 15

# 迷路のマップデータ。
MAP = [
    [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
    [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1],
    [1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1],
    [1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 1],
    [1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1],

    [1, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1],
    [1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1],
    [3, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 0, 3],
    [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1],
    [1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 1],

    [1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1, 1, 1, 0, 0, 0, 1],
    [1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1],
    [1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1],
    [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1],
    [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1]
]

CHIPSIZE = 32

MAPWIDTH = 20
MAPHEIGHT = 15

class Sprite(pygame.sprite.Sprite):
    def __init__(self, filename, x, y,framex=None,framey=None):
        pygame.sprite.Sprite.__init__(self)
        self.image = image_load(filename)
        # 画像を分割するかどうかをチェックする。
        if (framex and framey) is None:
            width = self.image.get_width()
            height = self.image.get_height()
            transColor = self.image.get_at((0, 0))
            self.image.set_colorkey(transColor, RLEACCEL)
        else:
            self.imageList = split_image_load(self.image,framex,framey)
            width = self.imageList[0].get_width()
            height = self.imageList[0].get_height()
            transColor = self.imageList[0].get_at((0, 0))
            self.imageList[0].set_colorkey(transColor, RLEACCEL)

        self.rect = Rect(x, y, width, height)

        # 浮動小数点に変換
        self.fpx = float(self.rect.x)
        self.fpy = float(self.rect.y)
        # 移動量
        self.fpvx = 0.0
        self.fpvy = 0.0

class Player(Sprite):
    def __init__(self):
        Sprite.__init__(self, "PacMan.bmp", 300, 256,32,32)
        self.freamnum = 0
        self.temp_key = 0
        self.dir_num = 0
        self.walk_flag = False

    def change_speed(self, x, y):
        self.fpvx = x
        self.fpvy = y

    def update(self,blocks):
        self.blocks = blocks
        pressed_key = pygame.key.get_pressed()

        if pressed_key[K_RIGHT]:
            self.walk_flag = True
            self.temp_key = 1
        elif pressed_key[K_LEFT]:
            self.walk_flag = True
            self.temp_key = 2
        elif pressed_key[K_UP]:
            self.walk_flag = True
            self.temp_key = 4
        elif pressed_key[K_DOWN]:
            self.walk_flag = True
            self.temp_key = 8
        else:
            self.temp_key = 0

        self.Move()

        self.dir_change()

        # self.rectを更新するついでに整数座標に戻します。
        self.rect.x = int(self.fpx)
        self.rect.y = int(self.fpy)

        sys.stdout.flush()

    def dir_change(self):
        if self.temp_key == 1:
            self.dir_num = RIGHT
        if self.temp_key == 2:
            self.dir_num = LEFT
        if self.temp_key == 4:
            self.dir_num = UP
        if self.temp_key == 8:
            self.dir_num = DOWN

    def Move(self):
        if self.walk_flag:
            if self.dir_num == RIGHT:
                self.freamnum = RIGHT
                self.change_speed(2, 0)
            if self.dir_num == LEFT:
                self.freamnum = LEFT
                self.change_speed(-2,0)
            if self.dir_num == UP:
                self.freamnum = UP
                self.change_speed(0,-2)
            if self.dir_num == DOWN:
                self.freamnum = DOWN
                self.change_speed(0,2)

            self.fpx += self.fpvx
            self.fpy += self.fpvy

            x = int(self.fpx)
            y = int(self.fpy)

            if self.Is_Able_To_Go(x,y,MAP) is not True:
                self.walk_flag = False
        else:
            self.change_speed(0,0)

    def Is_Able_To_Go(self,x,y,data):
        if self.dir_num == UP:
            if data[int((y/32)-1)][int(x/32)] == 1:
                return False
        if self.dir_num == LEFT:
            if data[int(y/32)][int((x/32)-1)] == 1:
                return False
        if self.dir_num == DOWN:
            if data[int((y/32)+1)][int(x/32)] == 1:
                return False
        if self.dir_num == RIGHT:
            if data[int(y/32)][int((x/32)+1)] == 1:
                return False

        return True

    def draw(self, screen):
        self.playerImg = self.imageList[self.freamnum]
        screen.blit(self.playerImg, self.rect)

# モーションの構造体
class Motion:
    def __init__(self,_no,_time):
        self.no = _no
        self.time = _time

# プレイヤーのモーションの辞書。
PlayerMotion = {
    "RIGHT": 0,
    "DOWN": 1,
    "LEFT": 2,
    "UP": 3,
    "STOP": 4
}

MLOOP = -1
MSTOP = -2
MEND = -3

# アニメーションのクラス
class Animation:
    def __init__(self,MotionList):
        self.Motion = MotionList
        self.AnimCnt = 0.0
        self.MType = 0
        self.MIndex = 0

    def update(self):
        self.AnimCnt += 0.4
        if self.AnimCnt > self.Motion[self.MType][self.MIndex].no:
            self.AnimCnt = 0.0
            self.MIndex += 1
            if self.Motion[self.MType][self.MIndex].no == MLOOP:
                self.Mindex = 0
            elif self.Motion[self.MType][self.MIndex].no == MSTOP:
                self.Mindex -= 1
            elif self.Motion[self.MType][self.MIndex].no == MEND:
                # アクションゲーム用。
                pass
            else:
                pass

    def MotionChange(self,motion):
        self.MIndex = 0
        self.AnimCnt = 0.0
        self.MType = motion

# 画像を読み込む
def image_load(filename):
    try:
        image = pygame.image.load(filename)
    except pygame.error:
        print("Cannot load image:" % filename)
        raise SystemExit
    image = image.convert()
    return image

def split_image_load(image, frame_x, frame_y):
    # 画像のサイズ判定。
    size_x = image.get_width()
    size_y = image.get_height()

    imageList = []

    for j in range(0, size_y, frame_y):
        for i in range(0, size_x, frame_x):
            surface = pygame.Surface((frame_x, frame_y))
            surface.blit(image, (0, 0), (i, j, frame_x, frame_y))
            surface.set_colorkey(surface.get_at((0, 0)), RLEACCEL)
            surface.convert(surface)
            imageList.append(surface)

    return imageList

class Block(pygame.sprite.Sprite):
    def __init__(self,image,pos,screen):
        self.image = image
        pygame.sprite.Sprite.__init__(self)
        self.rect = self.image.get_rect()
        self.rect.topleft = pos
        screen.blit(self.image,pos)

class Map:
    def __init__(self):
        self.image = image_load("PacBlock.bmp")
        self.imageList = split_image_load(self.image, 32, 32)

    def create(self, screen,blocks):
        transColor = self.image.get_at((0, 0))
        self.image.set_colorkey(transColor, RLEACCEL)
        for y in range(MAP_HEIGHT):
            for x in range(MAP_WIDTH):
                if MAP[y][x] == BLOCK:
                    cell = Block(self.imageList[1], (x * CHIPSIZE, y * CHIPSIZE), screen)
                    blocks.add(cell)
                elif MAP[y][x] == FOOD:
                    screen.blit(self.imageList[2], (x * CHIPSIZE, y * CHIPSIZE))
                elif MAP[y][x] == WARP:
                    # ワープトンネルの座標を教える。
                    pass
                else:
                    pass

class Warp:
    pass

# メイン関数。
def Main():
    pygame.init()
    screen = pygame.display.set_mode((640, 480))
    player = pygame.sprite.Group()
    blocks = pygame.sprite.OrderedUpdates()
    pacman = Player()
    maze = Map()
    player.add(pacman)
    clock = pygame.time.Clock()
    # 迷路を生成する。
    maze.create(screen,blocks)
    # メインループ
    while True:
        clock.tick(60)

        screen.fill((0, 0, 0))

        blocks.draw(screen)

        player.update(blocks)

        pacman.draw(screen)

        pygame.display.update()  # 画面を更新

        # イベント処理
        for event in pygame.event.get():
            if event.type == QUIT:  # 閉じるボタンが押されたら終了
                pygame.quit()  # Pygameの終了(画面閉じられる)
                sys.exit()


if __name__ == '__main__':
    Main()
